#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Tests for Owner Class
"""

''' Tests for pyCatSim.api.human.Owner

Naming rules:
1. class: Test{filename}{Class}{method} with appropriate camel case
2. function: test_{method}_t{test_id}
Notes on how to test:
0. Make sure [pytest](https://docs.pytest.org) has been installed: `pip install pytest`
1. execute `pytest {directory_path}` in terminal to perform all tests in all testing files inside the specified directory
2. execute `pytest {file_path}` in terminal to perform all tests in the specified file
3. execute `pytest {file_path}::{TestClass}::{test_method}` in terminal to perform a specific test class/method inside the specified file
4. after `pip install pytest-xdist`, one may execute "pytest -n 4" to test in parallel with number of workers specified by `-n`
5. for more details, see https://docs.pytest.org/en/stable/usage.html
'''

import pytest
from pyCatSim import Cat, Owner

class TesthumanOwnerInit:
    ''' Test for Owner instantiation '''
     
    def test_init_t0(self):
         cat1 = Cat(name="Whiskers")
         owner1 = Owner(name="Sasha", cats_owned=cat1)

         assert owner1.name == 'Sasha'
         assert type(owner1.cats_owned) is list
         assert len(owner1.cats_owned) == 1
    
    def test_init_t1(self):
        cat1 = Cat(name="Whiskers")
        cat2 = Cat(name="Boots", color="tabby")
        # Multiple cats
        owner1 = Owner(name="Liam", cats_owned=[cat1, cat2])

        assert owner1.name == 'Liam'
        assert type(owner1.cats_owned) is list
        assert len(owner1.cats_owned) == 2


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
The human module controls the behavior of humans around cats
"""

from ..api.cat import Cat

class Owner:
    """
    Represents a cat owner who can care for one or more cats.

    Parameters
    ----------
    name : str
        The name of the owner.
    cats_owned : Cat or list of Cat
        A single Cat instance or a list of Cat instances representing the cats this owner is responsible for.

    Attributes
    ----------
    name : str
        The name of the owner.
    cats_owned : list of Cat
        The list of Cat objects owned by this person.

    Raises
    ------
    TypeError
        If cats_owned is neither a Cat nor a list of Cat objects.

    Examples
    --------
    .. jupyter-execute::
    
        from pyCatSim import Cat, Owner

        cat1 = Cat(name="Whiskers")
        cat2 = Cat(name="Boots", color="tabby")

        # Single cat
        owner1 = Owner(name="Sasha", cats_owned=cat1)

        # Multiple cats
        owner2 = Owner(name="Liam", cats_owned=[cat1, cat2])

        print(owner1.name)
        print([cat.name for cat in owner2.cats_owned])

    """
    def __init__(self, name, cats_owned):

        if isinstance(cats_owned, Cat):
            cats_owned = [cats_owned]
        elif isinstance(cats_owned, list):
            if not all(isinstance(cat, Cat) for cat in cats_owned):
                raise TypeError("All elements in cats_owned must be instances of Cat.")
        else:
            raise TypeError("cats_owned must be a Cat instance or a list of Cat instances.")

        self.name = name
        self.cats_owned = cats_owned



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
The cat module allows to create a Cat or a group of Cats (i.e. a Clowder)
"""


from ..utils import noises

import difflib

class Cat:
    
    """
    Represents a virtual cat with attributes like name, age, color, mood, hunger, energy, and health.
    
    Parameters
    ----------
    name : str
        The name of the cat.
    age : int, optional
        The age of the cat in years. Default is None.
    color : str, optional
        Coat color of the cat. Acceptable values are:
        'tabby', 'black', 'orange', 'tortoiseshell', and 'tuxedo'.
        Fuzzy matching is used to interpret close inputs. Default is None.
    mood : int, optional
        Mood level on a scale from -10 (grumpy) to 10 (ecstatic). Default is 0.
    hunger_level : int, optional
        Hunger level of the cat. Higher values indicate greater hunger. Default is 0.
    energy : int, optional
        Energy level of the cat. Default is 0.
    health : int, optional
        Health level of the cat. Default is 0.

    Attributes
    ----------
    name : str
        The name of the cat.
    age : int or None
        The age of the cat.
    color : str or None
        The interpreted or validated color of the cat.
    mood : int
        The cat's mood.
    hunger_level : int
        The cat's hunger level.
    energy : int
        The cat's energy level.
    health : int
        The cat's health level.
    
    
    Examples
    --------
    
    .. jupyter-execute::
        
        import pyCatSim as cats
        nutmeg = cats.Cat(name='Nutmeg', age = 3, color = 'tortoiseshell')
    
    """
    
    def __init__(self, name, age=None, color=None, mood=0, hunger_level=0, 
                 energy=0, health=0):
        
        
        self.name = name
        self.age = age
        
        possible_colors = ['tabby', 'black', 'orange', 'tortoiseshell', 'tuxedo']

        if color:
            color_normalized = color.lower().strip()
            match = difflib.get_close_matches(color_normalized, possible_colors, n=1, cutoff=0.6)

            if match:
                self.color = match[0]
                print(f"Color '{color}' interpreted as '{self.color}'.")
            else:
                print(f"Invalid color '{color}'. Valid options are: {', '.join(possible_colors)}.")
                self.color = None
        
        self.mood = mood
        self.hunger_level = hunger_level
        self.energy = energy
        self.health = health
    
    def make_noise(self, noise='meow', play=False):
        """
        

        Parameters
        ----------
        noise : string, optional
            The sound the cat makes. Valid options include "meow", "purr". The default is 'meow'.
        play : bool, optional
            Whether to play the sound (True) or print out the sound (False). The default is False.

        Raises
        ------
        ValueError
            Raises an error if the sound is not valid

        Returns
        -------
        str
            The sound
        
        See also
        --------
        
        pyCatSim.utils.noises.meow: Simulates a cat meow
        
        pyCatSim.utils.noises.purr: Simulates a cat purr
        
        Examples
        --------
        
        .. jupyter-execute::
            
            import pyCatSim as cats
            nutmeg = cats.Cat(name='Nutmeg', age = 3, color = 'tortoiseshell')
            nutmeg.make_noise()

        """
        
        noise_func ={
            'meow':noises.meow,
            'purr':noises.purr}
    
        if noise in noise_func.keys():
            return noise_func[noise](play=play)
        else:
            raise ValueError(f"Invalid noise '{noise}'. Valid options: {', '.join(noise_func.keys())}")
        
        
    def play(self, mood_boost=1, hunger_boost=1, energy_boost=-1):
            
            """
            Simulates playtime with the cat.
        
            Parameters
            ----------
            mood_boost : int, optional
                How much mood improves from play. Must be an integer. Default is 1.
            hunger_boost : int, optional
                How much hunger increases from play. Must be a positive integer. Default is 1.
            energy_boost : int, optional
                How much energy decreases from play. Must be a negative integer. Default is -1.
        
            Raises
            ------
            TypeError
                If any of the arguments are not integers.
            ValueError
                If hunger_boost is not positive or energy_boost is not negative.
                
            Examples
            --------
            
            .. jupyter-execute::
                
                import pyCatSim as cats
                nutmeg = cats.Cat(name='Nutmeg', age = 3, color = 'tortoiseshell')
                nutmeg.play()
                
            
            """
            for arg_name, arg_value in {
                "mood_boost": mood_boost,
                "hunger_boost": hunger_boost,
                "energy_boost": energy_boost
            }.items():
                if not isinstance(arg_value, int):
                    raise TypeError(f"{arg_name} must be an integer.")
        
            if hunger_boost <= 0:
                raise ValueError("Cats always get hungry when playing! hunger_boost must be positive.")
            if energy_boost >= 0:
                raise ValueError("Cats always get tired when playing! energy_boost must be negative.")
        
            self.mood += mood_boost
            self.hunger_level += hunger_boost
            self.energy += energy_boost
                        
        
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Tests for Cat Class
"""

''' Tests for pyCatSim.api.cat.Cat

Naming rules:
1. class: Test{filename}{Class}{method} with appropriate camel case
2. function: test_{method}_t{test_id}
Notes on how to test:
0. Make sure [pytest](https://docs.pytest.org) has been installed: `pip install pytest`
1. execute `pytest {directory_path}` in terminal to perform all tests in all testing files inside the specified directory
2. execute `pytest {file_path}` in terminal to perform all tests in the specified file
3. execute `pytest {file_path}::{TestClass}::{test_method}` in terminal to perform a specific test class/method inside the specified file
4. after `pip install pytest-xdist`, one may execute "pytest -n 4" to test in parallel with number of workers specified by `-n`
5. for more details, see https://docs.pytest.org/en/stable/usage.html
'''

import pytest
import sys
from pyCatSim import Cat

class TestcatCatInit:
    ''' Test for Cat instantiation '''
     
    def test_init_t0(self):
         cat = Cat(name="Boots", color="tabby")
         assert cat.name == 'Boots'
         assert cat.color == 'tabby'
    
    def test_init_t1(self):
        cat=Cat(name="Boots", age=2, color="tabby", mood=2, hunger_level=-1,
                energy = 2, health = 3)
        assert cat.name == 'Boots'
        assert cat.color == 'tabby'
        assert cat.age == 2
        assert cat.mood == 2
        assert cat.hunger_level == -1
        assert cat.energy == 2
        assert cat.health == 3

class TestcatCatNoise:
    ''' Test for Cat noise'''
    @pytest.mark.parametrize(('noise','play'),
                             [
                                 ('meow', False),
                                 ('meow', True),
                                 ('purr', False),
                                 ('purr', True)
                            ]
                            )
    def test_noise_t0(self,noise,play):
        if sys.platform == "linux" and play is True:
            pytest.skip("Skipping sound playback test on Linux.")

        cat = Cat(name="Boots", color="tabby")
        if play is True:
            cat.make_noise(noise,play)
        else:
            v = cat.make_noise(noise,play)
            if noise == 'meow':
                assert v == 'Meow!'
            elif noise == 'purr':
                assert v == 'Purrr'
    
    
    @pytest.mark.xfail
    def test_noise_t1(self, noise='speak'):
        cat = Cat(name="Boots", color="tabby")
        cat.make_noise()

class TestcatCatPlay:
    ''' Test for the play function'''
    
    def test_play_t0(self):
        cat=Cat(name="Boots", age=2, color="tabby", mood=2, hunger_level=-1,
                energy = 2, health = 3)
        
        cat.play()
        
        assert cat.mood == 3
        assert cat.hunger_level == 0
        assert cat.energy == 1  


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
This module contains possible sounds cats can make
"""


__all__=['meow',
         'purr']

from playsound import playsound
import os
from pathlib import Path 

# Path to the sound files
SOUND_DIR = Path(__file__).parents[1].joinpath("sounds").resolve()

def meow(play=False):
    """
    Simulates a meow

    Parameters
    ----------
    play : Bool, optional
        Whether to play the sound (True) or display the text (False). The default is False.

    Returns
    -------
    str
        If play is False, returns the sound as text

    """
    
    if play is False:
        return "Meow!"
    else:
        playsound(os.path.join(SOUND_DIR, "meow.mp3"))

def purr(play=False):
    """
    Simulates a purr

    Parameters
    ----------
    play : Bool, optional
        Whether to play the sound (True) or display the text (False). The default is False.

    Returns
    -------
    str
        If play is False, returns the sound as text

    """
    
    if play is False:
        return "Purrr"
    else:
        playsound(os.path.join(SOUND_DIR, "purr.mp3"))

    
